@@ -1,5 +1,7 @@
 #pragma once
 
+#include "../dxvk/dxvk_staging.h"
+
 #include "d3d11_buffer.h"
 #include "d3d11_texture.h"
 
@@ -16,16 +18,33 @@ namespace dxvk {
    * zero-initialization for buffers and images.
    */
   class D3D11Initializer {
-    constexpr static size_t MaxTransferMemory    = 32 * 1024 * 1024;
-    constexpr static size_t MaxTransferCommands  = 512;
+    // Use a staging buffer with a linear allocator to service small uploads
+    constexpr static VkDeviceSize StagingBufferSize = 1ull << 20;
   public:
 
+    // Maximum number of copy and clear commands to record before flushing
+    constexpr static size_t MaxCommandsPerSubmission = 512u;
+
+    // Maximum amount of staging memory to allocate before flushing
+    constexpr static size_t MaxMemoryPerSubmission = (env::is32BitHostPlatform() ? 12u : 48u) << 20;
+
+    // Maximum amount of memory in flight. If there are pending uploads while
+    // this limit is exceeded, further initialization will be stalled.
+    constexpr static size_t MaxMemoryInFlight = 3u * MaxMemoryPerSubmission;
+
     D3D11Initializer(
             D3D11Device*                pParent);
     
     ~D3D11Initializer();
 
-    void Flush();
+    void FlushCsChunk() {
+      std::lock_guard<dxvk::mutex> lock(m_csMutex);
+
+      if (!m_csChunk->empty())
+        FlushCsChunkLocked();
+    }
+
+    void NotifyContextFlush();
 
     void InitBuffer(
             D3D11Buffer*                pBuffer,
@@ -44,10 +63,14 @@ namespace dxvk {
 
     D3D11Device*      m_parent;
     Rc<DxvkDevice>    m_device;
-    Rc<DxvkContext>   m_context;
+    
+    DxvkStagingBuffer m_stagingBuffer;
+    Rc<sync::Fence>   m_stagingSignal;
 
     size_t            m_transferCommands  = 0;
-    size_t            m_transferMemory    = 0;
+
+    dxvk::mutex       m_csMutex;
+    DxvkCsChunkRef    m_csChunk;
 
     void InitDeviceLocalBuffer(
             D3D11Buffer*                pBuffer,
@@ -68,12 +91,30 @@ namespace dxvk {
     void InitTiledTexture(
             D3D11CommonTexture*         pTexture);
 
-    void FlushImplicit();
-    void FlushInternal();
+    void ThrottleAllocationLocked();
+
+    void ExecuteFlush();
+
+    void ExecuteFlushLocked();
 
     void SyncSharedTexture(
             D3D11CommonTexture*         pResource);
 
+    void FlushCsChunkLocked();
+
+    void NotifyContextFlushLocked();
+
+    template<typename Cmd>
+    void EmitCs(Cmd&& command) {
+      std::lock_guard<dxvk::mutex> lock(m_csMutex);
+
+      if (unlikely(!m_csChunk->push(command))) {
+        FlushCsChunkLocked();
+
+        m_csChunk->push(command);
+      }
+    }
+
   };
 
 }
\ No newline at end of file
