@@ -584,19 +584,22 @@ namespace dxvk {
     struct alignas(16) UboData {
       float colorMatrix[3][4];
       float coordMatrix[3][2];
+      VkRect2D srcRect;
       float yMin, yMax;
       VkBool32 isPlanar;
     };
 
-    D3D11ImmediateContext* m_ctx;
+    D3D11ImmediateContext*  m_ctx;
 
-    Rc<DxvkSampler> m_sampler;
-    Rc<DxvkShader> m_vs;
-    Rc<DxvkShader> m_fs;
-    Rc<DxvkBuffer> m_ubo;
+    Rc<DxvkDevice>          m_device;
+    Rc<DxvkShader>          m_vs;
+    Rc<DxvkShader>          m_fs;
+    Rc<DxvkBuffer>          m_ubo;
 
     VkExtent2D m_dstExtent = { 0u, 0u };
 
+    bool m_resourcesCreated = false;
+
     void ApplyColorMatrix(float pDst[3][4], const float pSrc[3][4]);
 
     void ApplyYCbCrMatrix(float pColorMatrix[3][4], bool UseBt709);
@@ -608,6 +611,12 @@ namespace dxvk {
       const D3D11VideoProcessorStreamState* pStreamState,
       const D3D11_VIDEO_PROCESSOR_STREAM*   pStream);
 
+    void CreateUniformBuffer();
+
+    void CreateShaders();
+
+    void CreateResources();
+
     void UnbindResources();
 
   };
