@@ -32,12 +32,12 @@ namespace dxvk {
   };
 
   D3D8Device::D3D8Device(
-    D3D8Interface*                pParent,
-    Com<d3d9::IDirect3DDevice9>&& pDevice,
-    D3DDEVTYPE                    DeviceType,
-    HWND                          hFocusWindow,
-    DWORD                         BehaviorFlags,
-    D3DPRESENT_PARAMETERS*        pParams)
+        D3D8Interface*                pParent,
+        Com<d3d9::IDirect3DDevice9>&& pDevice,
+        D3DDEVTYPE                    DeviceType,
+        HWND                          hFocusWindow,
+        DWORD                         BehaviorFlags,
+        D3DPRESENT_PARAMETERS*        pParams)
     : D3D8DeviceBase(std::move(pDevice))
     , m_d3d8Options(pParent->GetOptions())
     , m_parent(pParent)
@@ -55,7 +55,7 @@ namespace dxvk {
 
     ResetState();
     RecreateBackBuffersAndAutoDepthStencil();
-    
+
     if (m_d3d8Options.batching)
       m_batcher = new D3D8Batcher(this, GetD3D9());
   }
@@ -73,7 +73,7 @@ namespace dxvk {
 
     HRESULT res;
     Com<d3d9::IDirect3DQuery9> pQuery;
-    
+
     switch (DevInfoID) {
       // pre-D3D8 queries
       case 0:
@@ -81,7 +81,7 @@ namespace dxvk {
       case D3DDEVINFOID_D3DTEXTUREMANAGER:
       case D3DDEVINFOID_TEXTURING:
         return E_FAIL;
-      
+
       case D3DDEVINFOID_VCACHE:
         // The query will return D3D_OK on Nvidia and D3DERR_NOTAVAILABLE on AMD/Intel
         // in D3D9, however in the case of the latter we'll need to return a
@@ -148,7 +148,7 @@ namespace dxvk {
     return GetD3D9()->GetAvailableTextureMem();
   }
 
-  HRESULT STDMETHODCALLTYPE D3D8Device::ResourceManagerDiscardBytes(DWORD bytes) { 
+  HRESULT STDMETHODCALLTYPE D3D8Device::ResourceManagerDiscardBytes(DWORD bytes) {
     return GetD3D9()->EvictManagedResources();
   }
 
@@ -165,7 +165,7 @@ namespace dxvk {
     HRESULT res = GetD3D9()->GetDeviceCaps(&caps9);
 
     if (likely(SUCCEEDED(res)))
-      dxvk::ConvertCaps8(caps9, pCaps);
+      ConvertCaps8(caps9, pCaps);
 
     return res;
   }
@@ -207,7 +207,7 @@ namespace dxvk {
 
     if (unlikely(pPresentationParameters == nullptr || ppSwapChain == nullptr))
       return D3DERR_INVALIDCALL;
-    
+
     Com<d3d9::IDirect3DSwapChain9> pSwapChain9;
     d3d9::D3DPRESENT_PARAMETERS params = ConvertPresentParameters9(pPresentationParameters);
     HRESULT res = GetD3D9()->CreateAdditionalSwapChain(
@@ -429,7 +429,7 @@ namespace dxvk {
 
     if (likely(SUCCEEDED(res)))
       *ppVertexBuffer = ref(new D3D8VertexBuffer(this, std::move(pVertexBuffer9), Pool, Usage));
-  
+
     return res;
   }
 
@@ -446,10 +446,10 @@ namespace dxvk {
 
     Com<d3d9::IDirect3DIndexBuffer9> pIndexBuffer9 = nullptr;
     HRESULT res = GetD3D9()->CreateIndexBuffer(Length, Usage, d3d9::D3DFORMAT(Format), d3d9::D3DPOOL(Pool), &pIndexBuffer9, NULL);
-    
+
     if (likely(SUCCEEDED(res)))
       *ppIndexBuffer = ref(new D3D8IndexBuffer(this, std::move(pIndexBuffer9), Pool, Usage));
-    
+
     return res;
   }
 
@@ -570,7 +570,7 @@ namespace dxvk {
     res = src->LockRect(&srcLocked, &srcRect, D3DLOCK_READONLY);
     if (FAILED(res))
       return res;
-    
+
     res = dst->LockRect(&dstLocked, &dstRect, 0);
     if (FAILED(res)) {
       src->UnlockRect();
@@ -935,15 +935,31 @@ namespace dxvk {
     // SetDepthStencilSurface is a separate call
     D3D8Surface* zStencil = static_cast<D3D8Surface*>(pNewZStencil);
 
-    if (likely(m_depthStencil != zStencil)) {
-      StateChange();
-      res = GetD3D9()->SetDepthStencilSurface(D3D8Surface::GetD3D9Nullable(zStencil));
+    // Depth stencil dimensions can not be lower than
+    // those of the currently set render target.
+    if (m_renderTarget != nullptr && zStencil != nullptr) {
+      D3DSURFACE_DESC rtDesc;
+      res = m_renderTarget->GetDesc(&rtDesc);
 
       if (unlikely(FAILED(res))) return res;
 
-      m_depthStencil = zStencil;
+      D3DSURFACE_DESC dsDesc;
+      res = zStencil->GetDesc(&dsDesc);
+
+      if (unlikely(FAILED(res))) return res;
+
+      if (unlikely(dsDesc.Width  < rtDesc.Width
+                || dsDesc.Height < rtDesc.Height))
+        return D3DERR_INVALIDCALL;
     }
 
+    StateChange();
+    res = GetD3D9()->SetDepthStencilSurface(D3D8Surface::GetD3D9Nullable(zStencil));
+
+    if (unlikely(FAILED(res))) return res;
+
+    m_depthStencil = zStencil;
+
     return D3D_OK;
   }
 
@@ -996,7 +1012,7 @@ namespace dxvk {
   }
 
   HRESULT STDMETHODCALLTYPE D3D8Device::BeginScene() { return GetD3D9()->BeginScene(); }
-  
+
   HRESULT STDMETHODCALLTYPE D3D8Device::EndScene() { StateChange(); return GetD3D9()->EndScene(); }
 
   HRESULT STDMETHODCALLTYPE D3D8Device::Clear(
@@ -1099,7 +1115,7 @@ namespace dxvk {
   HRESULT STDMETHODCALLTYPE D3D8Device::GetClipPlane(DWORD Index, float* pPlane) {
     return GetD3D9()->GetClipPlane(Index, pPlane);
   }
-  
+
   HRESULT STDMETHODCALLTYPE D3D8Device::CreateStateBlock(
           D3DSTATEBLOCKTYPE     Type,
           DWORD*                pToken) {
@@ -1388,7 +1404,7 @@ namespace dxvk {
     m_streams[0] = D3D8VBO {nullptr, 0};
     m_indices = nullptr;
     m_baseVertexIndex = 0;
-    
+
     return GetD3D9()->DrawIndexedPrimitiveUP(
       d3d9::D3DPRIMITIVETYPE(PrimitiveType),
       MinVertexIndex,
@@ -1474,12 +1490,12 @@ namespace dxvk {
 
     if (unlikely(StreamNumber >= d8caps::MAX_STREAMS))
       return D3DERR_INVALIDCALL;
-    
+
     const D3D8VBO& vbo = m_streams[StreamNumber];
 
     *ppStreamData = vbo.buffer.ref();
     *pStride      = vbo.stride;
-    
+
     return D3D_OK;
   }
 
@@ -1680,17 +1696,30 @@ namespace dxvk {
     if (unlikely(pDeclaration == nullptr || pHandle == nullptr))
       return D3DERR_INVALIDCALL;
 
+    // Validate VS version for non-FF shaders
+    if (pFunction != nullptr) {
+      uint32_t majorVersion = (pFunction[0] >> 8) & 0xff;
+      uint32_t minorVersion = pFunction[0] & 0xff;
+
+      if (unlikely(majorVersion != 1 || minorVersion > 1)) {
+        Logger::err(str::format("D3D8Device::CreateVertexShader: Unsupported VS version ", majorVersion, ".", minorVersion));
+        return D3DERR_INVALIDCALL;
+      }
+    }
+
     D3D8VertexShaderInfo& info = m_vertexShaders.emplace_back();
 
     // Store D3D8 bytecodes in the shader info
-    if (pDeclaration != nullptr)
-      for (UINT i = 0; pDeclaration[i+1] != D3DVSD_END(); i++)
-        info.declaration.push_back(pDeclaration[i]);
+    for (UINT i = 0; pDeclaration[i] != D3DVSD_END(); i++)
+      info.declaration.push_back(pDeclaration[i]);
+    info.declaration.push_back(D3DVSD_END());
 
-    if (pFunction != nullptr)
-      for (UINT i = 0; pFunction[i+1] != D3DVS_END(); i++)
+    if (pFunction != nullptr) {
+      for (UINT i = 0; pFunction[i] != D3DVS_END(); i++)
         info.function.push_back(pFunction[i]);
-    
+      info.function.push_back(D3DVS_END());
+    }
+
     D3D9VertexShaderCode result = TranslateVertexShader8(pDeclaration, pFunction, m_d3d8Options);
 
     // Create vertex declaration
@@ -1792,7 +1821,7 @@ namespace dxvk {
 
     /*
     // Slow path. Use to debug cached shader validation. //
-    
+
     d3d9::IDirect3DVertexShader9* pVertexShader;
     HRESULT res = GetD3D9()->GetVertexShader(&pVertexShader);
 
@@ -1829,6 +1858,9 @@ namespace dxvk {
 
       info->declaration.clear();
       info->function.clear();
+
+      if (m_currentVertexShader == Handle)
+        m_currentVertexShader = 0;
     }
 
     return D3D_OK;
@@ -1841,12 +1873,12 @@ namespace dxvk {
 
     if (unlikely(!pInfo))
       return D3DERR_INVALIDCALL;
-    
+
     UINT SizeOfData = *pSizeOfData;
-    
+
     // Get actual size
     UINT ActualSize = pInfo->declaration.size() * sizeof(DWORD);
-    
+
     if (pData == nullptr) {
       *pSizeOfData = ActualSize;
       return D3D_OK;
@@ -1854,8 +1886,10 @@ namespace dxvk {
 
     // D3D8-specific behavior
     if (SizeOfData < ActualSize) {
-      *pSizeOfData = ActualSize;
-      return D3DERR_MOREDATA;
+      // D3DERR_MOREDATA should be returned according to the D3D8 documentation,
+      // along with a correction to the ActualSize, however tests have shown that
+      // D3DERR_INVALIDCALL is returned and no size correction is performed.
+      return D3DERR_INVALIDCALL;
     }
 
     memcpy(pData, pInfo->declaration.data(), ActualSize);
@@ -1869,12 +1903,12 @@ namespace dxvk {
 
     if (unlikely(!pInfo))
       return D3DERR_INVALIDCALL;
-    
+
     UINT SizeOfData = *pSizeOfData;
-    
+
     // Get actual size
     UINT ActualSize = pInfo->function.size() * sizeof(DWORD);
-    
+
     if (pData == nullptr) {
       *pSizeOfData = ActualSize;
       return D3D_OK;
@@ -1882,8 +1916,10 @@ namespace dxvk {
 
     // D3D8-specific behavior
     if (SizeOfData < ActualSize) {
-      *pSizeOfData = ActualSize;
-      return D3DERR_MOREDATA;
+      // D3DERR_MOREDATA should be returned according to the D3D8 documentation,
+      // along with a correction to the ActualSize, however tests have shown that
+      // D3DERR_INVALIDCALL is returned and no size correction is performed.
+      return D3DERR_INVALIDCALL;
     }
 
     memcpy(pData, pInfo->function.data(), ActualSize);
@@ -1901,8 +1937,16 @@ namespace dxvk {
     if (unlikely(pFunction == nullptr || pHandle == nullptr))
       return D3DERR_INVALIDCALL;
 
+    uint32_t majorVersion = (pFunction[0] >> 8) & 0xff;
+    uint32_t minorVersion = pFunction[0] & 0xff;
+
+    if (unlikely(majorVersion != 1 || minorVersion > 4)) {
+      Logger::err(str::format("D3D8Device::CreatePixelShader: Unsupported PS version ", majorVersion, ".", minorVersion));
+      return D3DERR_INVALIDCALL;
+    }
+
     d3d9::IDirect3DPixelShader9* pPixelShader;
-    
+
     HRESULT res = GetD3D9()->CreatePixelShader(pFunction, &pPixelShader);
 
     if (likely(SUCCEEDED(res))) {
@@ -1915,7 +1959,7 @@ namespace dxvk {
   }
 
   inline d3d9::IDirect3DPixelShader9* getPixelShaderPtr(D3D8Device* device, DWORD Handle) {
-    
+
     Handle = getShaderIndex(Handle);
 
     if (unlikely(Handle >= device->m_pixelShaders.size())) {
@@ -1986,6 +2030,9 @@ namespace dxvk {
 
     m_pixelShaders[getShaderIndex(Handle)] = nullptr;
 
+    if (m_currentPixelShader == Handle)
+      m_currentPixelShader = 0;
+
     return D3D_OK;
   }
 
@@ -1998,11 +2045,11 @@ namespace dxvk {
       return D3DERR_INVALIDCALL;
 
     UINT SizeOfData = *pSizeOfData;
-    
+
     // Get actual size
     UINT ActualSize = 0;
     pPixelShader->GetFunction(nullptr, &ActualSize);
-    
+
     if (pData == nullptr) {
       *pSizeOfData = ActualSize;
       return D3D_OK;
@@ -2010,8 +2057,10 @@ namespace dxvk {
 
     // D3D8-specific behavior
     if (SizeOfData < ActualSize) {
-      *pSizeOfData = ActualSize;
-      return D3DERR_MOREDATA;
+      // D3DERR_MOREDATA should be returned according to the D3D8 documentation,
+      // along with a correction to the ActualSize, however tests have shown that
+      // D3DERR_INVALIDCALL is returned and no size correction is performed.
+      return D3DERR_INVALIDCALL;
     }
 
     return pPixelShader->GetFunction(pData, &SizeOfData);
