@@ -1,6 +1,7 @@
 #pragma once
 
 #include <vector>
+#include <mutex>
 
 #include "dxgi_adapter.h"
 #include "dxgi_monitor.h"
@@ -12,7 +13,13 @@ namespace dxvk {
 
   class DxgiFactory;
 
-  class DxgiVkFactory : public IDXGIVkInteropFactory {
+  struct DXVK_VK_GLOBAL_HDR_STATE {
+    uint32_t Serial;
+    DXGI_COLOR_SPACE_TYPE ColorSpace;
+    DXGI_VK_HDR_METADATA  Metadata;
+  };
+
+  class DxgiVkFactory : public IDXGIVkInteropFactory1 {
 
   public:
 
@@ -30,6 +37,14 @@ namespace dxvk {
             VkInstance*               pInstance,
             PFN_vkGetInstanceProcAddr* ppfnVkGetInstanceProcAddr);
 
+    HRESULT STDMETHODCALLTYPE GetGlobalHDRState(
+            DXGI_COLOR_SPACE_TYPE   *pOutColorSpace,
+            DXGI_HDR_METADATA_HDR10 *pOutMetadata);
+
+    HRESULT STDMETHODCALLTYPE SetGlobalHDRState(
+            DXGI_COLOR_SPACE_TYPE    ColorSpace,
+      const DXGI_HDR_METADATA_HDR10 *pMetadata);
+
   private:
 
     DxgiFactory* m_factory;
@@ -173,6 +188,8 @@ namespace dxvk {
     DxgiMonitorInfo* GetMonitorInfo() {
       return &m_monitorInfo;
     }
+
+    DXVK_VK_GLOBAL_HDR_STATE GlobalHDRState();
     
   private:
     
@@ -182,9 +199,15 @@ namespace dxvk {
     DxgiMonitorInfo  m_monitorInfo;
     UINT             m_flags;
     BOOL             m_monitorFallback;
-    
-    HWND m_associatedWindow = nullptr;
-    
+      
+
+    HRESULT STDMETHODCALLTYPE CreateSwapChainForHwndBase(
+            IUnknown*             pDevice,
+            HWND                  hWnd,
+      const DXGI_SWAP_CHAIN_DESC1* pDesc,
+      const DXGI_SWAP_CHAIN_FULLSCREEN_DESC* pFullscreenDesc,
+            IDXGIOutput*          pRestrictToOutput,
+            IDXGISwapChain1**     ppSwapChain);
   };
   
 }
