@@ -813,7 +813,7 @@ namespace dxvk {
     if (!selectedMode.RefreshRate.Denominator)
       selectedMode.RefreshRate.Denominator = 1;
 
-    if (!wsi::setWindowMode(outputDesc.Monitor, m_window, ConvertDisplayMode(selectedMode)))
+    if (!wsi::setWindowMode(outputDesc.Monitor, m_window, &m_windowState, ConvertDisplayMode(selectedMode)))
       return DXGI_ERROR_NOT_CURRENTLY_AVAILABLE;
 
     DXGI_VK_MONITOR_DATA* monitorData = nullptr;
@@ -997,6 +997,27 @@ namespace dxvk {
     if (m_presenter2 == nullptr)
       return;
 
+    // Engage the frame limiter with large sync intervals even in windowed
+    // mode since we want to avoid double-presenting to the swap chain.
+    if (SyncInterval != m_frameRateSyncInterval && m_descFs.Windowed) {
+      m_frameRateSyncInterval = SyncInterval;
+      m_frameRateRefresh = 0.0f;
+
+      if (SyncInterval > 1 && wsi::isWindow(m_window)) {
+        wsi::WsiMode mode = { };
+
+        if (wsi::getCurrentDisplayMode(wsi::getWindowMonitor(m_window), &mode)) {
+          if (mode.refreshRate.numerator && mode.refreshRate.denominator) {
+            m_frameRateRefresh = double(mode.refreshRate.numerator)
+                               / double(mode.refreshRate.denominator);
+          }
+        }
+      }
+    } else if (!m_descFs.Windowed) {
+      // Reset tracking when in fullscreen mode
+      m_frameRateSyncInterval = 0;
+    }
+
     // Use a negative number to indicate that the limiter should only
     // be engaged if the target frame rate is actually exceeded
     double frameRate = std::max(m_frameRateOption, 0.0);
