@@ -92,6 +92,14 @@ namespace dxvk {
       return m_vkd;
     }
     
+    /**
+     * \brief Vulkan instance functions
+     * \returns Vulkan instance functions
+     */
+    Rc<vk::InstanceFn> vki() const {
+      return m_instance->vki();
+    }
+    
     /**
      * \brief Logical device handle
      * \returns The device handle
@@ -100,6 +108,14 @@ namespace dxvk {
       return m_vkd->device();
     }
 
+    /**
+     * \brief Checks whether debug functionality is enabled
+     * \returns \c true if debug utils are enabled
+     */
+    bool isDebugEnabled() const {
+      return bool(m_instance->extensions().extDebugUtils);
+    }
+
     /**
      * \brief Device options
      * \returns Device options
