@@ -6,6 +6,7 @@ namespace dxvk {
   constexpr VkColorComponentFlags RGB  = VK_COLOR_COMPONENT_R_BIT | VK_COLOR_COMPONENT_G_BIT | VK_COLOR_COMPONENT_B_BIT;
   constexpr VkColorComponentFlags RG   = VK_COLOR_COMPONENT_R_BIT | VK_COLOR_COMPONENT_G_BIT;
   constexpr VkColorComponentFlags R    = VK_COLOR_COMPONENT_R_BIT;
+  constexpr VkColorComponentFlags A    = VK_COLOR_COMPONENT_A_BIT;
 
   const std::array<DxvkFormatInfo, DxvkFormatCount> g_formatInfos = {{
     // VK_FORMAT_UNDEFINED
@@ -568,14 +569,21 @@ namespace dxvk {
       DxvkFormatFlag::MultiPlane, VkExtent3D { 1, 1, 1 },
       { DxvkPlaneFormatInfo { 1, { 1, 1 } },
         DxvkPlaneFormatInfo { 2, { 2, 2 } } } },
+
+    // VK_FORMAT_A1B5G5R5_UNORM_PACK16_KHR
+    { 2, RGBA, VK_IMAGE_ASPECT_COLOR_BIT },
+
+    // VK_FORMAT_A8_UNORM_KHR
+    { 1, A, VK_IMAGE_ASPECT_COLOR_BIT },
   }};
   
   
-  const std::array<std::pair<VkFormat, VkFormat>, 4> g_formatGroups = {{
+  const std::array<std::pair<VkFormat, VkFormat>, 5> g_formatGroups = {{
     { VK_FORMAT_UNDEFINED,                  VK_FORMAT_BC7_SRGB_BLOCK            },
     { VK_FORMAT_G8B8G8R8_422_UNORM_KHR,     VK_FORMAT_B8G8R8G8_422_UNORM_KHR    },
     { VK_FORMAT_A4R4G4B4_UNORM_PACK16,      VK_FORMAT_A4B4G4R4_UNORM_PACK16     },
-    { VK_FORMAT_G8_B8_R8_3PLANE_420_UNORM,   VK_FORMAT_G8_B8R8_2PLANE_420_UNORM  },
+    { VK_FORMAT_G8_B8_R8_3PLANE_420_UNORM,  VK_FORMAT_G8_B8R8_2PLANE_420_UNORM  },
+    { VK_FORMAT_A1B5G5R5_UNORM_PACK16_KHR,  VK_FORMAT_A8_UNORM_KHR              },
   }};
   
   
