@@ -20,6 +20,21 @@ namespace dxvk {
   };
 
 
+  /**
+   * \brief Instance creation flags
+   *
+   * These flags will be passed to the app version field of the Vulkan
+   * instance, so that drivers can adjust behaviour for some edge cases
+   * that are not implementable with Vulkan itself.
+   */
+  enum class DxvkInstanceFlag : uint32_t {
+    /** Enforce D3D9 behaviour for texture coordinate snapping */
+    ClientApiIsD3D9,
+  };
+
+  using DxvkInstanceFlags = Flags<DxvkInstanceFlag>;
+
+
   /**
    * \brief DXVK instance
    * 
@@ -33,14 +48,17 @@ namespace dxvk {
 
     /**
      * \brief Creates new Vulkan instance
+     * \param [in] flags Instance flags
      */
-    DxvkInstance();
+    explicit DxvkInstance(DxvkInstanceFlags flags);
 
     /**
      * \brief Imports existing Vulkan instance
+     * \param [in] flags Instance flags
      */
     explicit DxvkInstance(
-      const DxvkInstanceImportInfo& args);
+      const DxvkInstanceImportInfo& args,
+            DxvkInstanceFlags       flags);
 
     ~DxvkInstance();
     
@@ -126,6 +144,18 @@ namespace dxvk {
     const DxvkInstanceExtensions& extensions() const {
       return m_extensions;
     }
+
+   /**
+    * \brief Instance extension name list
+    * 
+    * Returns the list of extensions that the
+    * instance was created with, provided by
+    * both DXVK and any extension providers.
+    * \returns Instance extension name list
+    */
+    const DxvkNameList& extensionNameList() const {
+      return m_extensionNames;
+    }
     
   private:
 
@@ -135,6 +165,8 @@ namespace dxvk {
     Rc<vk::LibraryFn>       m_vkl;
     Rc<vk::InstanceFn>      m_vki;
     DxvkInstanceExtensions  m_extensions;
+    DxvkNameSet             m_extensionSet;
+    DxvkNameList            m_extensionNames;
 
     VkDebugUtilsMessengerEXT m_messenger = VK_NULL_HANDLE;
 
@@ -145,7 +177,8 @@ namespace dxvk {
       const DxvkInstanceImportInfo& args);
 
     void createInstanceLoader(
-      const DxvkInstanceImportInfo& args);
+      const DxvkInstanceImportInfo& args,
+            DxvkInstanceFlags       flags);
 
     std::vector<DxvkExt*> getExtensionList(
             DxvkInstanceExtensions& ext,
