@@ -7,7 +7,20 @@ namespace dxvk {
   : m_device(device), m_callback(callback),
     m_submitThread([this] () { submitCmdLists(); }),
     m_finishThread([this] () { finishCmdLists(); }) {
+    auto vk = m_device->vkd();
 
+    VkSemaphoreTypeCreateInfo semaphoreType = { VK_STRUCTURE_TYPE_SEMAPHORE_TYPE_CREATE_INFO };
+    semaphoreType.semaphoreType = VK_SEMAPHORE_TYPE_TIMELINE;
+
+    VkSemaphoreCreateInfo semaphoreInfo = { VK_STRUCTURE_TYPE_SEMAPHORE_CREATE_INFO, &semaphoreType };
+
+    VkResult vrGraphics = vk->vkCreateSemaphore(vk->device(), &semaphoreInfo, nullptr, &m_semaphores.graphics);
+    VkResult vrTransfer = vk->vkCreateSemaphore(vk->device(), &semaphoreInfo, nullptr, &m_semaphores.transfer);
+
+    if (vrGraphics || vrTransfer) {
+      throw DxvkError(str::format("Failed to create timeline semaphores: ",
+        vrGraphics > vrTransfer ? vrGraphics : vrTransfer));
+    }
   }
   
   
@@ -23,6 +36,9 @@ namespace dxvk {
 
     m_submitThread.join();
     m_finishThread.join();
+
+    vk->vkDestroySemaphore(vk->device(), m_semaphores.graphics, nullptr);
+    vk->vkDestroySemaphore(vk->device(), m_semaphores.transfer, nullptr);
   }
   
   
@@ -37,7 +53,6 @@ namespace dxvk {
     entry.status = status;
     entry.submit = std::move(submitInfo);
 
-    m_pending += 1;
     m_submitQueue.push(std::move(entry));
     m_appendCond.notify_all();
   }
@@ -74,6 +89,19 @@ namespace dxvk {
   }
 
 
+  void DxvkSubmissionQueue::waitForIdle() {
+    std::unique_lock<dxvk::mutex> lock(m_mutex);
+
+    m_submitCond.wait(lock, [this] {
+      return m_submitQueue.empty();
+    });
+
+    m_finishCond.wait(lock, [this] {
+      return m_finishQueue.empty();
+    });
+  }
+
+
   void DxvkSubmissionQueue::lockDeviceQueue() {
     m_mutexQueue.lock();
 
@@ -107,44 +135,52 @@ namespace dxvk {
       lock.unlock();
 
       // Submit command buffer to device
-      VkResult status = VK_NOT_READY;
-
       if (m_lastError != VK_ERROR_DEVICE_LOST) {
         std::lock_guard<dxvk::mutex> lock(m_mutexQueue);
 
         if (m_callback)
           m_callback(true);
 
-        if (entry.submit.cmdList != nullptr)
-          status = entry.submit.cmdList->submit();
-        else if (entry.present.presenter != nullptr)
-          status = entry.present.presenter->presentImage();
+        if (entry.submit.cmdList != nullptr) {
+          entry.result = entry.submit.cmdList->submit(m_semaphores, m_timelines);
+          entry.timelines = m_timelines;
+        } else if (entry.present.presenter != nullptr) {
+          entry.result = entry.present.presenter->presentImage(entry.present.presentMode, entry.present.frameId);
+        }
 
         if (m_callback)
           m_callback(false);
       } else {
         // Don't submit anything after device loss
         // so that drivers get a chance to recover
-        status = VK_ERROR_DEVICE_LOST;
+        entry.result = VK_ERROR_DEVICE_LOST;
       }
 
       if (entry.status)
-        entry.status->result = status;
+        entry.status->result = entry.result;
       
       // On success, pass it on to the queue thread
       lock = std::unique_lock<dxvk::mutex>(m_mutex);
 
-      if (status == VK_SUCCESS) {
-        if (entry.submit.cmdList != nullptr)
-          m_finishQueue.push(std::move(entry));
-      } else if (status == VK_ERROR_DEVICE_LOST || entry.submit.cmdList != nullptr) {
-        Logger::err(str::format("DxvkSubmissionQueue: Command submission failed: ", status));
-        m_lastError = status;
-        m_device->waitForIdle();
+      bool doForward = (entry.result == VK_SUCCESS) ||
+        (entry.present.presenter != nullptr && entry.result != VK_ERROR_DEVICE_LOST);
+
+      if (doForward) {
+        m_finishQueue.push(std::move(entry));
+      } else {
+        Logger::err(str::format("DxvkSubmissionQueue: Command submission failed: ", entry.result));
+        m_lastError = entry.result;
+
+        if (m_lastError != VK_ERROR_DEVICE_LOST)
+          m_device->waitForIdle();
       }
 
       m_submitQueue.pop();
       m_submitCond.notify_all();
+
+      // Good time to invoke allocator tasks now since we
+      // expect this to get called somewhat periodically.
+      m_device->m_objects.memoryManager().performTimedTasks();
     }
   }
   
@@ -152,6 +188,8 @@ namespace dxvk {
   void DxvkSubmissionQueue::finishCmdLists() {
     env::setThreadName("dxvk-queue");
 
+    auto vk = m_device->vkd();
+
     while (!m_stopped.load()) {
       std::unique_lock<dxvk::mutex> lock(m_mutex);
 
@@ -172,31 +210,52 @@ namespace dxvk {
       DxvkSubmitEntry entry = std::move(m_finishQueue.front());
       lock.unlock();
       
-      VkResult status = m_lastError.load();
-      
-      if (status != VK_ERROR_DEVICE_LOST)
-        status = entry.submit.cmdList->synchronizeFence();
-      
-      if (status != VK_SUCCESS) {
-        m_lastError = status;
-        m_device->waitForIdle();
+      if (entry.submit.cmdList != nullptr) {
+        VkResult status = m_lastError.load();
+
+        if (status != VK_ERROR_DEVICE_LOST) {
+          std::array<VkSemaphore, 2> semaphores = { m_semaphores.graphics, m_semaphores.transfer };
+          std::array<uint64_t, 2> timelines = { entry.timelines.graphics, entry.timelines.transfer };
+
+          VkSemaphoreWaitInfo waitInfo = { VK_STRUCTURE_TYPE_SEMAPHORE_WAIT_INFO };
+          waitInfo.semaphoreCount = semaphores.size();
+          waitInfo.pSemaphores = semaphores.data();
+          waitInfo.pValues = timelines.data();
+
+          status = vk->vkWaitSemaphores(vk->device(), &waitInfo, ~0ull);
+        }
+
+        if (status != VK_SUCCESS) {
+          m_lastError = status;
+
+          if (status != VK_ERROR_DEVICE_LOST)
+            m_device->waitForIdle();
+        }
+      } else if (entry.present.presenter != nullptr) {
+        // Signal the frame and then immediately destroy the reference.
+        // This is necessary since the front-end may want to explicitly
+        // destroy the presenter object. 
+        entry.present.presenter->signalFrame(entry.result,
+          entry.present.presentMode, entry.present.frameId);
+        entry.present.presenter = nullptr;
       }
 
       // Release resources and signal events, then immediately wake
       // up any thread that's currently waiting on a resource in
       // order to reduce delays as much as possible.
-      entry.submit.cmdList->notifyObjects();
+      if (entry.submit.cmdList != nullptr)
+        entry.submit.cmdList->notifyObjects();
 
       lock.lock();
-      m_pending -= 1;
-
       m_finishQueue.pop();
       m_finishCond.notify_all();
       lock.unlock();
 
       // Free the command list and associated objects now
-      entry.submit.cmdList->reset();
-      m_device->recycleCommandList(entry.submit.cmdList);
+      if (entry.submit.cmdList != nullptr) {
+        entry.submit.cmdList->reset();
+        m_device->recycleCommandList(entry.submit.cmdList);
+      }
     }
   }
   
