@@ -43,6 +43,11 @@ namespace dxvk {
     { R"(\\Wow(Classic)?\.exe$)", {{
       { "dxvk.hideIntegratedGraphics",      "True"  },
     }} },
+    /* Bright Memory - Will choose other vendors   *
+     * over Intel even if Intel is the only dGPU   */
+    { R"(\\BrightMemory_EP1-Win64-Shipping\.exe$)", {{
+      { "dxvk.hideIntegratedGraphics",      "True"  },
+    }} },
 
     /**********************************************/
     /* D3D11 GAMES                                */
@@ -455,6 +460,21 @@ namespace dxvk {
     { R"(\\x86_64\\Warcraft III\.exe$)", {{
       { "dxvk.hideIntegratedGraphics",      "True"  },
     }} },
+    /* Earth Defense Force 5                       */
+    { R"(\\EDF5\.exe$)", {{
+      { "dxgi.tearFree",                    "False" },
+      { "dxgi.syncInterval",                "1"     },
+    }} },
+    /* The Hurricane of the Varstray               *
+     * Too fast above 60fps                        */
+    { R"(\\Varstray_steam(_demo)?\.exe$)", {{
+      { "dxgi.maxFrameRate",                "60" },
+    }} },
+    /* Far Cry 5 and New Dawn                      *
+     * Invisible terrain on Intel                  */
+    { R"(\\FarCry(5|NewDawn)\.exe$)", {{
+      { "d3d11.zeroInitWorkgroupMemory",    "True" },
+    }} },
 
     /**********************************************/
     /* D3D9 GAMES                                 */
@@ -469,15 +489,23 @@ namespace dxvk {
     { R"(\\anarchyonline\.exe$)", {{
       { "d3d9.memoryTrackTest",             "True" },
     }} },
-    /* Borderlands 2 and The Pre Sequel!           */
-    { R"(\\Borderlands(2|PreSequel)\.exe$)", {{
-      { "d3d9.lenientClear",                "True" },
-      { "d3d9.supportDFFormats",            "False" },
-    }} },
     /* Borderlands                                */
     { R"(\\Borderlands\.exe$)", {{
       { "d3d9.lenientClear",                "True" },
     }} },
+    /* Borderlands 2                               *
+     * Missing lava in Vault of the Warrior        *
+     * without Strict floats                       */
+    { R"(\\Borderlands2\.exe$)", {{
+      { "d3d9.lenientClear",                "True" },
+      { "d3d9.supportDFFormats",            "False" },
+      { "d3d9.floatEmulation",              "Strict" },
+    }} },
+    /* Borderlands: The Pre-Sequel                  */
+    { R"(\\BorderlandsPreSequel\.exe$)", {{
+      { "d3d9.lenientClear",                "True" },
+      { "d3d9.supportDFFormats",            "False" },
+    }} },
     /* Gothic 3                                   */
     { R"(\\Gothic(3|3Final| III Forsaken Gods)\.exe$)", {{
       { "d3d9.supportDFFormats",            "False" },
@@ -492,7 +520,8 @@ namespace dxvk {
        The Sims Life Stories,
        The Sims Pet Stories,
        and The Sims Castaway Stories             */
-    { R"(\\(Sims2.*|TS2BodyShop|SimsLS|SimsPS|SimsCS)\.exe$)", {{
+    { R"(\\(Sims2.*|TS2BodyShop|SimsLS|SimsPS|SimsCS)"
+      R"(|The Sims 2 Content Manager|TS2HomeCrafterPlus)\.exe$)", {{
       { "d3d9.customVendorId",              "10de" },
       { "d3d9.customDeviceId",              "0091" },
       { "d3d9.customDeviceDesc",            "GeForce 7800 GTX" },
@@ -644,11 +673,6 @@ namespace dxvk {
     { R"(\\FarCry\.exe$)", {{
       { "d3d9.customVendorId",              "10de" },
     }} },
-    /* Earth Defense Force 5 */
-    { R"(\\EDF5\.exe$)", {{
-      { "dxgi.tearFree",                    "False" },
-      { "dxgi.syncInterval",                "1"     },
-    }} },
     /* Sine Mora EX */
     { R"(\\SineMoraEX\.exe$)", {{
       { "d3d9.maxFrameRate",                "60" },
@@ -819,8 +843,8 @@ namespace dxvk {
      * Main menu won't render after intros     *
      * and CPU bound performance               */
     { R"(\\(PANZERS|PANZERS_Phase_2)\.exe$)", {{
-      { "d3d9.enableDialogMode",            "True"   },
-      { "d3d9.cachedDynamicBuffers",        "True"   },
+      { "d3d9.deferSurfaceCreation",        "True" },
+      { "d3d9.cachedDynamicBuffers",        "True" },
     }} },
     /* DC Universe Online                      *
      * Freezes after alt tabbing               */
@@ -998,9 +1022,19 @@ namespace dxvk {
       { "d3d9.customVendorId",              "10de" },
       { "d3d9.cachedDynamicBuffers",        "True" },
     }} },
-    /* Alpha Protocol - Rids unwanted reflections  */
-    { R"(\\APGame\.exe$)", {{
-      { "d3d9.forceSamplerTypeSpecConstants", "True" },
+    /* Dark Sector - Crashes in places             */
+    { R"(\\DS\.exe$)", {{
+      { "d3d9.textureMemory",                "0" },
+    }} },
+    /* Arcana Heart 3 Love Max + Xtend version     *
+     * Game speed is too fast above 60 fps         */
+    { R"(\\(AH3LM|AALib)\.exe$)", {{
+      { "d3d9.maxFrameRate",                "60" },
+    }} },
+    /* May Payne 3 - Visual issues on some drivers *
+     * such as ANV (and amdvlk when set to True)   */
+    { R"(\\MaxPayne3\.exe$)", {{
+      { "d3d9.floatEmulation",              "Strict" },
     }} },
 
     /**********************************************/
@@ -1011,11 +1045,6 @@ namespace dxvk {
     { R"(\\DukeForever\.exe$)", {{
       { "d3d9.maxFrameRate",                "60"   },
     }} },
-    /* Indiana Jones and the Emperor's Tomb      *
-     * Fixes intro window being stuck on screen  */
-    { R"(\\indy\.exe$)", {{
-      { "d3d9.enableDialogMode",            "True" },
-    }} },
     /* Anito: Defend a Land Enraged              */
     { R"(\\Anito\.exe$)", {{
       { "d3d9.memoryTrackTest",             "True" },
@@ -1052,7 +1081,6 @@ namespace dxvk {
     /* Need for Speed III: Hot Pursuit           *
        (with the "Modern Patch")                 */
     { R"(\\nfs3\.exe$)", {{
-      { "d3d9.enableDialogMode",            "True" },
       { "d3d9.cachedDynamicBuffers",        "True" },
       { "d3d8.batching",                    "True" },
     }} },
@@ -1061,7 +1089,6 @@ namespace dxvk {
        Won't actually render anything in game     *
        without a memory limit in place            */
     { R"(\\nfs4\.exe$)", {{
-      { "d3d9.enableDialogMode",            "True" },
       { "d3d9.cachedDynamicBuffers",        "True" },
       { "d3d9.memoryTrackTest",             "True" },
       { "d3d9.maxAvailableMemory",           "256" },
@@ -1495,6 +1522,8 @@ namespace dxvk {
     }
 
     if (!confLine.empty()) {
+      ctx.active = true;
+
       // Inform the user that we parsing config from environment, might
       // help when debugging configuration issues
       Logger::info(str::format("Found config env: ", confLine));
