@@ -28,7 +28,7 @@ namespace dxvk {
      * \brief Sets target frame rate
      * \param [in] frameRate Target frame rate
      */
-    void setTargetFrameRate(double frameRate);
+    void setTargetFrameRate(double frameRate, uint32_t maxLatency);
 
     /**
      * \brief Stalls calling thread as necessary
@@ -48,14 +48,15 @@ namespace dxvk {
 
     TimerDuration   m_targetInterval  = TimerDuration::zero();
     TimePoint       m_nextFrame       = TimePoint();
+    uint32_t        m_maxLatency      = 0;
 
     bool            m_envOverride     = false;
 
-    uint32_t                  m_heuristicFrameCount = 0;
-    std::array<TimePoint, 16> m_heuristicFrameTimes = { };
-    bool                      m_heuristicEnable = false;
+    uint32_t        m_heuristicFrameCount = 0;
+    TimePoint       m_heuristicFrameTime  = TimePoint();
+    bool            m_heuristicEnable     = false;
 
-    bool testRefreshHeuristic(TimerDuration interval, TimePoint now);
+    bool testRefreshHeuristic(TimerDuration interval, TimePoint now, uint32_t maxLatency);
 
   };
 
